#!/bin/bash

# Usage info
show_help() {
cat << EOF
Usage: ${0##*/} [-h] [-m MEMORY] [XPIPFILE]...
Launch X!TandemPipeline on XPIPFILE. With no XPIPFILE, launches X!TandemPipeline with a new empty project.

    -h          display this help and exit
    -m MEMORY   the required memory in Mbytes (3000 for 3Gb)
EOF
}

# Initialize our own variables:
memory="1500"
jvm="/usr/bin/java"

jvmxmx="-Xmx2500m"

OPTIND=1 # Reset is necessary if getopts was used previously in the script.  It is a good idea to make this local in a function.
while getopts "hm:" opt; do
    case "$opt" in
        h)
            show_help
            exit 0
            ;;
        m)  memory=$OPTARG
            jvmxmx = "-Xmx${memory}m"
            ;;
        '?')
            show_help >&2
            exit 1
            ;;
    esac
done
shift "$((OPTIND-1))" # Shift off the options and optional --.

#printf 'java -Xmx%sm -Xms256m \n' "$memory"
#printf '<%s>\n' "$@"

SCRIPT_PATH=$(dirname `which $0`)

$jvm $jvmxmx -Xms256m -cp "${SCRIPT_PATH}/bin/*:${SCRIPT_PATH}/lib/*:${SCRIPT_PATH}/lib/swt/swt-gtk-linux-x86_64.jar:${SCRIPT_PATH}/lib/batik/*:${SCRIPT_PATH}/lib/http/*:${SCRIPT_PATH}:." fr.inra.pappso.xtandempipeline.XtandemPipelineMain $@

# End of file
