package fr.inra.pappso.xtandempipeline;

/*
 * Copyright © 2011, Benoit Valot, Olivier Langella

 This file is part of XtandemPipeline.

 XtandemPipeline is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 XtandemPipeline is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with XtandemPipeline.  If not, see <http://www.gnu.org/licenses/>.

 */

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

import fr.inra.pappso.xtandempipeline.class_msms.XpipFile;
import fr.inra.pappso.xtandempipeline.class_msms.config;
import fr.inra.pappso.xtandempipeline.ui.swt.Main_Windows;

public class XtandemPipelineMain {
	private static final Logger logger = Logger
			.getLogger(XtandemPipelineMain.class);

	/**
	 * @param args
	 * @return
	 * @return
	 */
	static public String version = "3.4.3";
	static public String versionName = "Elastine Durcie";

	public static String about = "X!Tandem Pipeline is developped by PAPPSO team (http://pappso.inra.fr).\n\n"
			+ "This software is distributed under the terms of the GNU General Public License.\n\n"
			+ "For more information on X!Tandem search tools, see http://www.thegpm.org/tandem/";

	public static String warning = "new mzIdentML X!Tandem export option";

	// public static String news =
	// "3.3.1 : Start and stop peptide positions added in exported spreadsheet files (ODS or CSV)";
	public static String news = "3.4.2 : new xslt export option";

	public static void main(String[] args) {

		File log4jPropertyFile = new File(config.getRacinePath()
				+ System.getProperty("file.separator") + "log4j.properties");
		if (log4jPropertyFile.exists()) {
			PropertyConfigurator.configure(log4jPropertyFile.getAbsolutePath());
			logger.info("loading log4j proterties from "
					+ log4jPropertyFile.getAbsolutePath());
			// return (false);
		} else {
			// Chargement en release mode
			if (System.getProperty("os.name").contains("Linux")) {
				PropertyConfigurator.configure(XtandemPipelineMain.class
						.getResource("/conf/log4j.linux.properties"));
				logger.info("Linux logging");
				// System.out.print("Linux logging");
			} else {
				PropertyConfigurator.configure(XtandemPipelineMain.class
						.getResource("/conf/log4j.windows.properties"));
				logger.info("Window logging");
				// System.out.print("Window logging");
			}
		}
		if (args.length > 0) {
			try {
				logger.debug("URL specified in aguments :" + args[0]);
				URL urlOfXpipData = new URL(args[0]);

				XpipFile in;
				try {
					urlOfXpipData.openConnection();

					if (args.length > 2) {
						XtandemPipelineSession.getInstance()
								.setPROTICportController(urlOfXpipData,
										args[1], args[2]);
					}
					// InputStream in = uc.getInputStream();
					in = new XpipFile(urlOfXpipData);
					logger.info("Starting new main with ProticPort");
					new Main_Windows(in);

				} catch (IOException e) {
					logger.error("unable to open URL "
							+ urlOfXpipData.toString());
					// Perhaps that the first argument is a local path to an
					// XPIP file ?
					in = new XpipFile(urlOfXpipData.toString());

					if (!in.exists()) {
						logger.error("XPIP file " + in.getAbsolutePath()
								+ " does not exists");
						in = null;
					}
					new Main_Windows(in);

				}
			} catch (MalformedURLException e) {
				logger.error("MalformedURLException on " + args[0]);
				// Perhaps that the first argument is a local path to an
				// XPIP file ?
				XpipFile in = new XpipFile(args[0]);

				if (!in.exists()) {
					logger.error("XPIP file " + in.getAbsolutePath()
							+ " does not exists");
					in = null;
				}
				new Main_Windows(in);
			}
		} else {
			logger.info("Starting new main in classic mode");
			new Main_Windows(null);
		}
	}
}
