/*******************************************************************************
 * Copyright (c) 2016 Olivier Langella <Olivier.Langella@u-psud.fr>
 *
 * This file is part of XTandemPipeline.
 *
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/
package fr.inra.pappso.xtandempipeline.class_msms.IdentificationDataFile;

import java.io.File;
import java.util.Hashtable;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import org.apache.log4j.Logger;

import fr.inra.pappso.libodsstream.OdsDocWriterInterface;
import fr.inra.pappso.xtandempipeline.Utils;
import fr.inra.pappso.xtandempipeline.XtandemPipelineSession;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.MzxmlFile;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;
import fr.inra.pappso.xtandempipeline.sax_parse.HandlerXtResultFile;
import fr.inra.pappso.xtandempipeline.sax_parse.HandlerXtResultFile4ProcessingParams;
import fr.inra.pappso.xtandempipeline.sax_parse.HandlerXtResultPhosphoFile;
import fr.inra.pappso.xtandempipeline.sax_parse.parse_file;
import fr.inra.pappso.xtandempipeline.sax_parse.HandlerXtResultFile4Spectrum;

public class IdentificationMzIdentMlFile extends IdentificationDataFile {
	private static final Logger logger = Logger
			.getLogger(IdentificationMzIdentMlFile.class);
	private Hashtable<Integer, Integer> scanNum2XtandemScanNum = new Hashtable<Integer, Integer>();

	public IdentificationMzIdentMlFile(File xtandemFile) throws MSMSException {
		super(xtandemFile);
	}

	@Override
	public Identification loadIdentification(
			Identification previousIdentification) throws Exception {
		HandlerXtResultFile tandem = new HandlerXtResultFile(
				previousIdentification, this);
		new parse_file(tandem, this.file);
		previousIdentification = tandem.get_identifications();
		return previousIdentification;
	}

	@Override
	public Identification loadPhosphoIdentification(
			Identification previousIdentification) throws Exception {
		HandlerXtResultPhosphoFile tandem = new HandlerXtResultPhosphoFile(
				previousIdentification, this);
		new parse_file(tandem, this.file);
		previousIdentification = tandem.get_identifications();
		return previousIdentification;
	}

	@Override
	public boolean loadMSMSSpectra(Peptide pep) throws Exception {
		HandlerXtResultFile4Spectrum handler = new HandlerXtResultFile4Spectrum(
				pep);
		logger.debug("scanNum2XtandemScanNum size "
				+ scanNum2XtandemScanNum.size());
		Integer scan = scanNum2XtandemScanNum.get(pep.get_scan());
		if (scan != null) {
			handler.setScanNumToFind(scan);
		} else {
			logger.error("no scan number found");
		}
		new parse_file(handler, this.file);
		return true;
	}

	public void writeProticDbMlDataProcessing(XMLStreamWriter writer)
			throws Exception {
		logger.debug("writeProticDbMlDataProcessing begin");
		HandlerXtResultFile4ProcessingParams handler = new HandlerXtResultFile4ProcessingParams();
		new parse_file(handler, this.file);

		writer.writeStartElement("dataProcessing");
		writer.writeStartElement("software");
		writer.writeStartElement("name");
		writer.writeAttribute("acc", "PROTICdbO:0000283");
		writer.writeCharacters("X!Tandem");
		writer.writeEndElement();// name
		writer.writeStartElement("version");
		writer.writeCharacters(handler.getXtandemVersionNumber());
		writer.writeEndElement();// version
		writer.writeEndElement();// software
		writer.writeStartElement("processingMethod");
		// mzXml source file name
		writeCvParam(writer, "PROTICdbO:0000343",
				handler.getMSMSDataSourceFileName(),
				"MS/MS data source file name");
		// model file name
		writeCvParam(writer, "PROTICdbO:0000342", handler.getXtModelFileName(),
				"X!tandem xml model file name");
		// xtandem result file name
		writeCvParam(writer, "PROTICdbO:0000341", this.file.getName(),
				"X!tandem xml result file name");
		writer.writeEndElement();// processingMethod
		writer.writeEndElement();// dataProcessing

	}

	private void writeCvParam(XMLStreamWriter writer, String acc, String value,
			String description) throws XMLStreamException {
		writer.writeStartElement("cvParam");

		writer.writeAttribute("value", value);
		if (description == null) {
			writer.writeAttribute("name", "N.A.");
		} else {
			writer.writeAttribute("name", description);

		}
		writer.writeAttribute("cvLabel", "PROTICdbO");
		writer.writeAttribute("accession", acc);
		writer.writeEndElement();// cvParam

	}

	@Override
	public void setScanCorrespondance(int fixedScanNum,
			int scanNumAsWrittenInDataSource) {
		if (fixedScanNum == scanNumAsWrittenInDataSource)
			return;
		logger.debug("setScanCorrespondance " + fixedScanNum + "=>"
				+ scanNumAsWrittenInDataSource);
		this.scanNum2XtandemScanNum.put(fixedScanNum,
				scanNumAsWrittenInDataSource);
	}

	@Override
	public Integer getScanCorrespondance(int scanNumber) {
		return scanNum2XtandemScanNum.get(scanNumber);
	}

	@Override
	public void writeOdsMsSampleInformations(OdsDocWriterInterface odsTable,
			boolean xtandem_info_extract, boolean mzxml_info_extract)
			throws MSMSException {
		logger.debug("writeOdsMsSampleInformations begin");
		if (xtandem_info_extract) {
			if (!this.file.exists()) {
				try {
					odsTable.writeCell("X!Tandem XML result file NOT FOUND");
					odsTable.writeCell(this.file.getAbsolutePath());
				} catch (Exception e) {
					String message = "ERROR writing ODS in writeOdsXtandemInformations "
							+ e.getMessage();
					logger.error(message);
					throw new MSMSException(message);
				}
			} else {
				HandlerXtResultFile4ProcessingParams handler = new HandlerXtResultFile4ProcessingParams();
				try {
					new parse_file(handler, this.file);
				} catch (Exception e) {
					String message = "ERROR parsing file "
							+ this.file.getName() + " " + e.getMessage();
					logger.error(message);
					throw new MSMSException(message);
				}

				try {

					// odsTable.writeCell("MS sample name");
					// odsTable.writeCell("X!tandem xml result file name");
					odsTable.writeCell(this.file.getName());
					// odsTable.writeCell("X!Tandem version");
					odsTable.writeCell(handler.getXtandemVersionNumber());

					// odsTable.writeCell("X!tandem xml model file name");
					odsTable.writeCell(handler.getXtModelFileName());
					// odsTable.writeCell("total spectra used");
					odsTable.writeCell(handler.getTotalSpectraUsed());
					// odsTable.writeCell("total spectra assigned");
					odsTable.writeCell(handler.getTotalSpectraAssigned());

					odsTable.writeCell(((double) handler.getTotalSpectraAssigned() / (double) handler
							.getTotalSpectraUsed()) * (double)100);
					// odsTable.writeCell("total unique assigned");
					odsTable.writeCell(handler.getTotalUniqueAssigned());
					// odsTable.writeCell("MS/MS data source file name");

					odsTable.writeCell(Utils.collectionToString(handler
							.getDatabaseFileList()));

					odsTable.writeCell(handler.getMSMSDataSourceFileName());

					if (mzxml_info_extract) {
						// odsTable.writeCell("MS/MS data source file path");
						MzxmlFile mzXml = handler.getMzxmlFile();
						if (mzXml != null) {
							odsTable.writeCell(mzXml.getAbsolutePath());
						}
						if (mzXml != null) {
							if (XtandemPipelineSession.getInstance()
									.getConfig()
									.get_export_expert_spreadsheet()) {
								mzXml.writeOdsInformations(odsTable);
							}
						}
					}

				} catch (Exception e) {
					String message = "ERROR writing ODS in writeOdsMsSampleInformations "
							+ e.getMessage();
					logger.error(message);
					throw new MSMSException(message);
				}
			}
		}
	}

	@Override
	public String getSampleName() throws MSMSException {
		return this.getName().replaceAll("\\.mzid$", "");
	}
}
