package fr.inra.pappso.xtandempipeline.class_msms;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.IdentificationDataFile.IdentificationDataSource;

public class MsRunSet extends HashSet<MsRun> {
	/**
	 *
	 */
	private static final long serialVersionUID = 1859166854395591445L;

	private static final Logger logger = Logger.getLogger(MsRunSet.class);

	private HashMap<String, MsRun> name2msRun = new HashMap<String, MsRun>(0);

	public MsRunSet() {
	}

	/**
	 * @param Take
	 *            name of sample, this is normally the filename without
	 *            extension
	 * @return MsRun instance
	 */
	public MsRun getInstanceBySampleName(String sampleName) {
		// Remove file extension for storing sample
		sampleName = sampleName.replaceAll("\\.xml$", "");
		sampleName = sampleName.replaceAll("\\.dat$", "");

		if (name2msRun.containsKey(sampleName)) {
			// logger.debug("get MsRun by sample name " + sampleName);
			return name2msRun.get(sampleName);
		}
		// logger.debug("add new MsRun with sample name " + sampleName);
		MsRun msRun = new MsRun(sampleName);
		name2msRun.put(sampleName, msRun);
		this.add(msRun);
		return msRun;
	}

	private MsRun getInstanceByIdentificationDataSource(
			IdentificationDataSource sampleFile) throws MSMSException {
		return getInstanceBySampleName(sampleFile.getSampleName());
	}

	/**
	 * @param Take
	 *            name of sample from file by removing extension.
	 * @return MsRun instance
	 * @throws MSMSException
	 */
	public MsRun getInstance(IdentificationDataSource sampleFile)
			throws MSMSException {
		// String sampleName = sampleFile.getName();
		MsRun msRun = getInstanceByIdentificationDataSource(sampleFile);
		sampleFile = msRun.addFile(sampleFile);
		return msRun;
	}

	public ArrayList<MsRun> getMsRunList() {
		ArrayList<MsRun> msRunList = new ArrayList<MsRun>(this);
		Collections.sort(msRunList);
		return (msRunList);
	}

	public boolean containsMsRunFromThisDataSource(
			IdentificationDataSource dataSourceToParse) throws MSMSException {
		String sampleName = dataSourceToParse.getSampleName();
		if (name2msRun.containsKey(sampleName)) {
			return true;
		}
		return false;
	}
}
