package fr.inra.pappso.xtandempipeline.class_msms;

import java.util.ArrayList;
import java.util.HashSet;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;
import fr.inra.pappso.xtandempipeline.grouping.interfaces.GrpPhosphoModif;

public class PhosphoIsland {

	/**
	 * 
	 */
	private static final Logger logger = Logger.getLogger(PhosphoIsland.class);

	private HashSet<Integer> peptideCover = new HashSet<Integer>();
	private HashSet<Integer> phosphoPosition = new HashSet<Integer>();

	public PhosphoIsland(Peptide peptide) throws Exception {
		if (!peptide.is_phopho_peptide()) {
			String error = "peptide "
					+ peptide.get_sample_scan_sequencepeptide_cle()
					+ " is not a phosphopeptide";
			logger.error(error);
			throw new MSMSException(error);
		}
		if (!peptide.isValid()) {
			String error = "peptide "
					+ peptide.get_sample_scan_sequencepeptide_cle()
					+ " is not valid";
			logger.error(error);
			throw new MSMSException(error);
		}

		for (int i = peptide.get_start(); i <= peptide.get_stop(); i++) {
			peptideCover.add(i);
		}
		for (GrpPhosphoModif mod : peptide.getGrpPhosphoModifs()) {
			phosphoPosition.add(mod.getPosi_in_prot());
		}
	}

	public boolean fusionWith(PhosphoIsland incomingGroup) throws Exception {
		// fusion with the incoming group if this contains common elements
		boolean fusion = false;

		for (Integer position : incomingGroup.peptideCover) {
			if (position == null) {
				String message = "position is null";
				logger.error(message);
				throw new Exception(message);
			}
			if (this.peptideCover.contains(position)) {
				fusion = true;
				this.peptideCover.addAll(incomingGroup.peptideCover);
				this.phosphoPosition.addAll(incomingGroup.phosphoPosition);
				break;
			}
		}

		return fusion;
	}

	public ArrayList<Integer> getPhosphoPositions() {
		// TODO Auto-generated method stub
		return new ArrayList<Integer>(this.phosphoPosition);
	}
}
