package fr.inra.pappso.xtandempipeline.class_msms;

/*******************************************************************************
 * Copyright (c) 2015 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 *
 * This file is part of XTandemPipeline.
 *
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contributors:
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr>
 ******************************************************************************/

import java.util.HashMap;

import org.apache.log4j.Logger;
import fr.inra.pappso.xtandempipeline.MsException.MSMSNotFoundException;

public class PsiMod {
	private static final Logger logger = Logger.getLogger(PsiMod.class);

	static HashMap<String, PsiMod> acc2mod = new HashMap<String, PsiMod>();

	private String accession;

	private String name;

	private String cvLabel;

	public static PsiMod getInstance(String accesion) throws MSMSNotFoundException {
		if (acc2mod.containsKey(accesion)) {
			return acc2mod.get(accesion);
		} else {
			PsiMod mod = new PsiMod(accesion);
			acc2mod.put(accesion, mod);
			return mod;

		}
	}

	private PsiMod(String accessionIn) throws MSMSNotFoundException {
		this.accession = accessionIn;
		cvLabel = "PSI-MOD";
		if (accession.equals("MOD:00719")) {
			this.name = "oxidation to L-methionine sulfoxide";
		} else if (accession.equals("MOD:00397")) {
			this.name = "iodoacetamide";
		} else if (accession.equals("MOD:00408")) {
			this.name = "N-acetylated residue";

		} else if (accession.equals("MOD:00696")) {
			this.name = "phosphorylated residue";
		} else if (accession.equals("MOD:01160")) {
			this.name = "loss of ammonia";
		} else if (accession.equals("MOD:00704")) {
			this.name = "dehydrated residue";
		} else if (accession.equals("MOD:00429")) {
			this.name = "dimethylated residue";
		} else if (accession.equals("MOD:00552")) {
			this.name = "4x(2)H labeled dimethyl residue";
		} else if (accession.equals("MOD:00628")) {
			this.name = "Hex2 DimethylpyrroleAdduct";
		} else if (accession.equals("MOD:00638")) {
			this.name = "2x(13)C,6x(2)H-dimethylated arginine";
		} else {
			String message = "accession not referenced " + accession;
			logger.error(message);
			throw new MSMSNotFoundException(message);
		}
	}

	public String getAccession() {
		return accession;
	}

	public String getName() {
		return name;
	}

	public String getCvLabel() {
		return this.cvLabel;
	}

}
