package fr.inra.pappso.xtandempipeline.class_msms;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.XtandemPipelineMain;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.XtandemFiles.XtandemModelFile;

public class config {

	private static final Logger logger = Logger.getLogger(config.class);

	private static String database_filter = "database_filter";
	// le fichier de la base de donnee pour laquelle on veux retirer les
	// protéines

	private static String protein_evalue = "protein_evalue";
	// log(evalue) maximun pour valider une protein

	private static String peptide_evalue = "peptide_evalue";
	// evalue maximun pour valider une protéine ou une isofome et pour combiner
	// les proteines

	private static String peptide_number = "peptide_number";
	// minimum de peptides differents permettant l'identification

	private static String mascot_evalue = "mascot_evalue";
	// Type of evalue stocked for mascot search theoritical or experimental

	// indicates that we want an export in ODS
	private static String export_ods = "export_ods";
	// indicates that we want an export in TXT
	private static String export_txt = "export_txt";

	private static String export_fasta = "export_fasta";
	// indique si l'on veux esporter les protéines identifiées en fichier Fasta

	private static String export_proticdbml = "export_proticdbml";
	// indique si l'on veux esporter les identification en fichier xml
	// compatible Protic

	private static String export_masschroq = "export_quantimscpp";

	private static String export_masschroqprm = "export_masschroqprm";
	// indique si l'on veux esporter les identification en fichier xml
	// compatible quantimscpp

	private static String protein_filter_to_complete = "protein_filter_to_complete";
	// indique si l'on veux appliquer les filtres au niveau proteique sur
	// l'ensemble des echantillons

	private static String export_fdr = "export_fdr";
	// indique si on veux exporter la répartition des peptides dans les banques
	// pour des evalue croissants

	private static String export_pepnovo = "export_pepnovo";
	// indique si on veux exporter les identifications pour une analyse en
	// pepnovo

	private static String xtandem_link = "xtandem_link";
	// lien vers le fichier X!tandem

	private static String xtandem_model = "xtandem_model";
	// lien vers les model xtandem

	private static String raw_data = "raw_data";
	// dossier contenant les raw_data

	private static String xmlresult = "xmlresult";
	// dossier contenant les fichiers resultats

	private static String protein_database = "protein_database";
	// dossier contenant les fichiers fasta

	private static String current_version = "current_version";
	// dossier contenant les fichiers fasta

	private Properties properties;

	private String condorSharedTmpDir = "condor_shared_tmp_dir";

	private String export_expert_spreadsheet = "export_expert_spreadsheet";

	private String export_raw_ms_sample_stat = "export_raw_ms_sample_stat";

	private static String condorSubmitPath = "condor_submit_path";

	private Float peptideProphetProbabilityThreshold = (float) 0.9;
	private Float peptideInterProphetProbabilityThreshold = (float) 0.9;

	public config() {
		properties = new Properties();
	}

	public File getXtandem_link() {
		File xtandem;
		if (System.getProperty("os.name").contains("Windows")) {
			xtandem = new File("C:/Benperl/Xtandem/tandem.exe");
		} else if (System.getProperty("os.name").contains("Linux")) {
			xtandem = new File("/usr/bin/tandem");
		} else {
			xtandem = new File("Not defined");
		}
		File file = new File(properties.getProperty(xtandem_link,
				xtandem.getPath()));
		return file;
	}

	public File getCondorSubmitCommand() {
		File condorSubmit = null;
		String path = properties.getProperty(condorSubmitPath, null);
		if (path == null) {
			return null;
		}
		condorSubmit = new File(path);
		if (condorSubmit.exists()) {
			if (condorSubmit.canExecute()) {
				return condorSubmit;
			}
		}
		return null;
	}

	public File getCondorQueueCommand() {
		File condorQueue = null;
		// String path = properties.getProperty(condorSubmitPath, null);
		// if (path == null) {
		// return null;
		// }
		condorQueue = new File("/usr/bin/condor_q");
		if (condorQueue.exists()) {
			if (condorQueue.canExecute()) {
				return condorQueue;
			}
		}
		return null;
	}

	public File getCondorSharedTmpDir() throws MSMSException {
		File condorSubmit = null;
		String path = properties.getProperty(condorSharedTmpDir, null);
		if (path == null) {
			return null;
		}
		condorSubmit = new File(path);
		if (condorSubmit.exists()) {
			return condorSubmit;
		}
		if (!(condorSubmit.mkdir())) {
			String message = "Could not create shared condor temp directory: "
					+ condorSubmit.getAbsolutePath();
			logger.error(message);
			throw new MSMSException(message);
		}
		return condorSubmit;

	}

	public void setXtandem_link(File F) {
		properties.setProperty(xtandem_link, F.getPath());
	}

	public File getXtandemModelsDirectory() {
		File file = new File(properties.getProperty(xtandem_model,
				config.getRacinePath() + System.getProperty("file.separator")
						+ "xtandem_models"));
		if (!file.exists())
			file.mkdirs();
		return (file);
	}

	public ArrayList<XtandemModelFile> getXtandem_models() {
		File[] files = this.getXtandemModelsDirectory().listFiles();
		ArrayList<XtandemModelFile> fi = new ArrayList<XtandemModelFile>();
		if (files == null) {
		} else {
			Arrays.sort(files);
			for (int i = 0; i < files.length; i++) {
				if (files[i].getName().endsWith(".xml"))
					fi.add(new XtandemModelFile(files[i]));
			}
		}
		logger.debug("ArrayList<XtandemModelFile>.size() " + fi.size());
		return (fi);
	}

	public XtandemModelFile getXtandemModelFile(String modelName) {
		modelName = modelName.replaceFirst("\\.xml$", "");
		modelName += ".xml";
		// File xml = this.get_current_xml(name + ".xml");
		ArrayList<XtandemModelFile> modelFiles = getXtandem_models();
		for (XtandemModelFile modelFile : modelFiles) {
			if (modelFile.getName().equalsIgnoreCase(modelName)) {
				return modelFile;
			}
		}
		// the X!Tandem model file does not exists : prepare a new file in the
		// model directory
		return new XtandemModelFile(new File(getXtandemModelsDirectory()
				.getAbsolutePath()
				+ System.getProperty("file.separator")
				+ modelName));
	}

	public FastaFileSet getprotein_databases() {
		File[] files = this.getProtein_database().listFiles();
		FastaFileSet fi = new FastaFileSet();
		if (files == null)
			return fi;
		Arrays.sort(files);
		for (int i = 0; i < files.length; i++) {
			if (files[i].getName().endsWith(".fasta"))
				fi.add(new FastaFile(files[i].getAbsolutePath()));
		}
		return (fi);
	}

	public void setXtandem_model(File F) {
		properties.setProperty(xtandem_model, F.getPath());
	}

	public File getRaw_data() {
		File file = new File(properties.getProperty(raw_data,
				System.getProperty("user.home")));
		if (file.exists()) {
			return (file);
		}
		return (new File(System.getProperty("user.home")));
	}

	public void setRaw_data(File F) {
		properties.setProperty(raw_data, F.getPath());
	}

	public File getXmlresult() {
		File file = new File(properties.getProperty(xmlresult,
				System.getProperty("user.home")));
		if (file.exists()) {
			return (file);
		}
		return (new File(System.getProperty("user.home")));
	}

	public void setXmlresult(File F) {
		properties.setProperty(xmlresult, F.getPath());
	}

	public File getProtein_database() {
		File file = new File(properties.getProperty(protein_database,
				System.getProperty("user.home")));
		if (file.exists()) {
			return (file);
		}
		return (new File(System.getProperty("user.home")));
	}

	public void setProtein_database(File F) {
		properties.setProperty(protein_database, F.getPath());
	}

	public void set_database_filter(File F) {
		properties.setProperty(database_filter, F.getPath());
	}

	public void set_protein_evalue(float F) {
		properties.setProperty(protein_evalue, "" + F);
	}

	public void set_peptide_evalue(float F) {
		properties.setProperty(peptide_evalue, "" + F);
	}

	public void set_peptide_number(int F) {
		properties.setProperty(peptide_number, "" + F);
	}

	public void set_mascot_evalue_theoritical(boolean F) {
		properties.setProperty(mascot_evalue, "" + F);
	}

	public void set_export_ods(boolean F) {
		logger.debug("set export_ods " + F);
		properties.setProperty(export_ods, "" + F);
	}

	public void set_export_txt(boolean F) {
		logger.debug("set export_txt " + F);
		properties.setProperty(export_txt, "" + F);
	}

	public void set_export_fasta(boolean F) {
		properties.setProperty(export_fasta, "" + F);
	}

	public void set_export_proticdbml(boolean F) {
		properties.setProperty(export_proticdbml, "" + F);
	}

	public void set_export_expert_spreadsheet(boolean F) {
		properties.setProperty(export_expert_spreadsheet, "" + F);
	}

	public void set_export_masschroq(boolean F) {
		properties.setProperty(export_masschroq, "" + F);
	}

	public void set_export_pepnovo(boolean F) {
		properties.setProperty(export_pepnovo, "" + F);
	}

	public void set_export_masschroqprm(boolean F) {
		properties.setProperty(export_masschroqprm, "" + F);
	}

	public void set_protein_filter_to_complete(boolean F) {
		properties.setProperty(protein_filter_to_complete, "" + F);
	}

	public FastaFile get_database_filter() {
		FastaFile file = new FastaFile(properties.getProperty(database_filter,
				"no selected"));
		return (file);
	}

	public float get_protein_evalue() {
		float value = Float.parseFloat(properties.getProperty(protein_evalue,
				"-4"));
		return (value);
	}

	public float get_peptide_evalue() {
		float value = Float.parseFloat(properties.getProperty(peptide_evalue,
				"0.05"));
		return (value);
	}

	public int get_peptide_number() {
		int value = Integer.parseInt(properties
				.getProperty(peptide_number, "2"));
		return (value);
	}

	public boolean get_mascot_evalue_theoritical() {
		String value = properties.getProperty(mascot_evalue, "true");
		if (value.equals("false"))
			return (false);
		return (true);
	}

	public boolean get_export_fasta() {
		String value = properties.getProperty(export_fasta, "false");
		if (value.equals("false"))
			return (false);
		return (true);
	}

	public boolean get_export_proticdbml() {
		String value = properties.getProperty(export_proticdbml, "false");
		if (value.equals("false"))
			return (false);
		return (true);
	}

	public boolean get_export_masschroq() {
		String value = properties.getProperty(export_masschroq, "false");
		if (value.equals("false"))
			return (false);
		return (true);
	}

	public boolean get_export_pepnovo() {
		String value = properties.getProperty(export_pepnovo, "false");
		if (value.equals("false"))
			return (false);
		return (true);
	}

	public boolean get_export_ods() {
		String value = properties.getProperty(export_ods, "true");
		if (value.equals("false"))
			return (false);
		return (true);
	}

	public boolean get_export_expert_spreadsheet() {
		String value = properties.getProperty(this.export_expert_spreadsheet,
				"false");
		if (value.equals("false"))
			return (false);
		return (true);
	}

	public boolean get_export_raw_ms_sample_stat_spreadsheet() {
		String value = properties.getProperty(this.export_raw_ms_sample_stat,
				"false");
		if (value.equals("false"))
			return (false);
		return (true);
	}


	public void set_export_raw_ms_sample_stat_spreadsheet(boolean F) {
		properties.setProperty(export_raw_ms_sample_stat, "" + F);
	}

	public boolean get_export_txt() {
		String value = properties.getProperty(export_txt, "false");
		if (value.equals("false"))
			return (false);
		return (true);
	}

	public boolean get_protein_filter_to_complete() {
		String value = properties.getProperty(protein_filter_to_complete,
				"false");
		if (value.equals("false"))
			return (false);
		return (true);
	}

	public void set_Export_fdr(boolean export) {
		properties.setProperty(export_fdr, "" + export);
	}

	public boolean get_Export_fdr() {
		String value = properties.getProperty(export_fdr, "false");
		if (value.equals("false"))
			return (false);
		return (true);
	}

	public boolean getXtandem_software_installed() {
		File file = this.getXtandem_link();
		if (!file.exists())
			return false;
		else if (!file.isFile())
			return false;
		else if (!file.canExecute())
			return false;
		else
			return true;
	}

	public boolean loadPropertiesFromXml() throws Exception {
		File xml = new File(config.getRacinePath()
				+ System.getProperty("file.separator") + "xtandempipeline.conf");
		if (!xml.exists()) {
			logger.info("New configuration file");
			return (false);
		}
		logger.info("Load properties configuration from : " + xml.getPath());
		properties.loadFromXML(new FileInputStream(xml));
		return (true);
	}

	public void savePropertiestoXml() throws Exception {
		File xml = new File(config.getRacinePath()
				+ System.getProperty("file.separator") + "xtandempipeline.conf");
		logger.info("Save properties condfiguration to : " + xml.getPath());
		properties.storeToXML(new FileOutputStream(xml),
				"Xtandem pipeline configuration file");
	}

	static public File getRacinePath() {
		File path;
		String XTP_CONFIG_HOME = System.getenv("XTP_CONFIG_HOME");
		if (XTP_CONFIG_HOME == null) {
			if (System.getProperty("os.name").contains("Linux")) {
				path = new File(System.getProperty("user.home")
						+ "/.config/xtandempipeline");
			} else {
				path = new File(System.getProperty("user.home")
						+ System.getProperty("file.separator")
						+ "xtandempipeline");
			}
		} else {
			path = new File(XTP_CONFIG_HOME);
		}
		if (!path.exists()) {
			path.mkdirs();
		}
		logger.debug("XTP_CONFIG_HOME = " + path.getAbsolutePath());
		return (path);
	}

	public void setCurrent_version() {
		properties.setProperty(current_version, XtandemPipelineMain.version);
	}

	public String getCurrent_version() {
		return properties.getProperty(current_version, "3.0.2");
	}

	/**
	 * get the version number of the X!Tandem binary currently installed
	 *
	 * @return version number
	 * @throws MSMSException
	 */
	public String getXtandemVersion() throws MSMSException {
		if (this.getXtandem_software_installed()) {
			String version = null;
			try {
				// X! TANDEM CYCLONE (2013.02.01.1)
				// X! TANDEM Sledgehammer (2013.09.01.1)
				Pattern patt = Pattern.compile("X! TANDEM [A-Z]+ \\((.*)\\)",
						Pattern.CASE_INSENSITIVE);
				Process pr = null;
				pr = Runtime.getRuntime().exec(
						this.getXtandem_link().getAbsolutePath());

				OutputStream out = pr.getOutputStream();
				out.write("input text".getBytes());
				out.close();
				BufferedReader in = new BufferedReader(new InputStreamReader(
						pr.getInputStream()));
				String line;
				while ((line = in.readLine()) != null) {
					Matcher m = patt.matcher(line);
					if (m.matches()) {
						version = m.group(1);
					}
				}

				pr.waitFor();

				in.close();

			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			if ((version == null) || (version.isEmpty())) {
				String message = "unable to get X!Tandem software version";
				logger.error(message);
				throw new MSMSException(message);
			}
			return version;
		} else {
			// X!tandem is not installed
			return null;
		}
	}

	private boolean checkMinimumProba(String Fstr) throws MSMSException {
		boolean ok = false;
		float fl = (float) 1.0;
		try {
			fl = Float.valueOf(Fstr).floatValue();
		} catch (Exception e) {
			throw new MSMSException(
					"Minimum probability MUST be a number between 0 and 1");
		}
		if (fl > 1) {
			throw new MSMSException("Minimum probability can not exceed 1");
		}
		if (fl < 0) {
			throw new MSMSException(
					"Minimum probability MUST be a number between 0 and 1");
		}
		return ok;
	}

	public void setMinimumPeptideProphetProbability(String Fstr)
			throws MSMSException {
		checkMinimumProba(Fstr);
		properties.setProperty("minimum_peptide_prophet_probability", Fstr);
	}

	public Float getMinimumPeptideProphetProbability() {
		float value = Float.parseFloat(properties.getProperty(
				"minimum_peptide_prophet_probability",
				peptideProphetProbabilityThreshold.toString()));
		return (value);
	}

	public void setMinimumPeptideInterProphetProbability(String Fstr)
			throws MSMSException {
		checkMinimumProba(Fstr);
		properties.setProperty("minimum_peptide_inter_prophet_probability",
				Fstr);
	}

	public Float getMinimumPeptideInterProphetProbability() {
		float value = Float.parseFloat(properties.getProperty(
				"minimum_peptide_inter_prophet_probability",
				peptideInterProphetProbabilityThreshold.toString()));
		return (value);
	}

	public boolean get_export_masschroqprm() {
		String value = properties.getProperty(export_masschroqprm, "false");
		if (value.equals("false"))
			return (false);
		return (true);
	}

	public int getXtandemMinimumMemoryInMb() {
		int value = Integer.parseInt(properties.getProperty(
				"xtandem_minimum_memory_in_mb", "3000"));
		return (value);
	}

}
