/*******************************************************************************
 * Copyright (c) 2013 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 *
 * This file is part of XTandemPipeline.
 *
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contributors:
 *     Benoit Valot <Benoit.Valot@moulon.inra.fr>
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr>
 ******************************************************************************/

package fr.inra.pappso.xtandempipeline.output.SpreadSheets;

import java.io.IOException;
import java.io.OutputStream;
import java.util.HashSet;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.stream.XMLStreamException;

import org.apache.log4j.Logger;

import fr.inra.pappso.libodsstream.OdsDocWriter;
import fr.inra.pappso.libodsstream.OdsDocWriterInterface;
import fr.inra.pappso.xtandempipeline.Utils;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.Match;
import fr.inra.pappso.xtandempipeline.class_msms.Protein;
import fr.inra.pappso.xtandempipeline.class_msms.GroupingProtein.PeptideMass;
import fr.inra.pappso.xtandempipeline.class_msms.GroupingProtein.PeptideMassSet;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;
import fr.inra.pappso.xtandempipeline.grouping.Group;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSetProt;
import fr.inra.pappso.xtandempipeline.grouping.SubGroup;

public class OdsPeptidePosOutput {
	private static final Logger logger = Logger
			.getLogger(OdsPeptidePosOutput.class);

	protected OdsDocWriterInterface odsTable;

	protected OdsPeptidePosOutput() {

	}

	public OdsPeptidePosOutput(OutputStream outputStream) throws MSMSException {
		try {
			odsTable = new OdsDocWriter(outputStream);
			odsTable.writeSheet("peptides_pos");
		} catch (IOException e) {
			String message = "IOException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (XMLStreamException e) {
			String message = "XMLStreamException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (DatatypeConfigurationException e) {
			String message = "DatatypeConfigurationException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	public OdsPeptidePosOutput(OdsDocWriterInterface odsTable)
			throws MSMSException {
		try {
			this.odsTable = odsTable;
			odsTable.writeSheet("peptides_pos");
		} catch (XMLStreamException e) {
			String message = "XMLStreamException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	public void close() throws MSMSException {
		try {
			odsTable.close();
		} catch (Exception e) {
			String message = "Exception closing OdsPeptidePosOutput "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	private void writeHeaders(Identification ident) throws MSMSException {
		try {
			// Peptide ID Protein Id Protein accession Protein description
			// Sequence Modifs Start Stop MH+ theo

			odsTable.writeLine();
			odsTable.writeCell("Peptide ID");
			odsTable.writeCell("Protein ID");
			odsTable.writeCell("accession");
			odsTable.writeCell("description");
			odsTable.writeCell("Sequence");
			odsTable.writeCell("Modifs");
			odsTable.writeCell("Start");
			odsTable.writeCell("Stop");
			odsTable.writeCell("MH+ theo");
			// odsTable.writeCell("Delta-ppm");
		} catch (XMLStreamException e) {
			String message = "XMLStreamException in writeHeaders() "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception in writeHeaders() " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}

	}

	public void write(Identification ident) throws MSMSException {
		try {

			this.writeHeaders(ident);
			for (Group group : ident.getGrouping().getGroupList()) {
				PeptideMassSet pepMassSet = new PeptideMassSet();

				logger.debug("pepMassSet size : " + pepMassSet.size());

				logger.debug("pepMassSet numbering");
				pepMassSet.numbering(group);

				for (SubGroup sg : group.getSubGroupSet().getSubGroupList()) {

					int k = 0;

					for (HashSampleScanSetProt hashProt : sg
							.getHashSampleScanSetProtList()) {
						k++;
						HashSet<PeptideMass> uniquePepMass = new HashSet<PeptideMass>();

						for (Peptide peptide : ((Match) hashProt
								.getGrpProtMatch()).get_valid_peptides()) {
							PeptideMass pepMass = pepMassSet
									.getInstance(peptide);
							if (uniquePepMass.add(pepMass)) {
								this.writeOnePeptideMassPerProt(ident, group,
										sg, k, pepMass, hashProt, peptide);
							}

						}
					}
				}

			}
			odsTable.writeLine();
			odsTable.writeLine();

		} catch (XMLStreamException e) {
			String message = "XMLStreamException in write(Identification ident) "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception in write(Identification ident) "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	private void writeOnePeptideMassPerProt(Identification ident, Group group,
			SubGroup sg, int k, PeptideMass pepMass,
			HashSampleScanSetProt hashProt, Peptide peptideThisProt)
			throws MSMSException {
		try {
			logger.debug("writeOnePeptideMassPerProt begin " + group + pepMass);
			Protein prot = (Protein) hashProt.getGrpProtMatch().getGrpProtein();
			// Peptide bestPeptide = pepMass.getBestEvaluePeptide();

			odsTable.writeLine();
			// odsTable.writeCell("Peptide ID");
			odsTable.writeCell(Utils.getPappsoPeptideMassId(group, pepMass));
			// odsTable.writeCell("Protein ID");
			odsTable.writeCell(Utils.getPappsoGroupingNumber(group, sg, k));
			// odsTable.writeCell("accession");
			odsTable.writeCell(prot.getAccession());
			// odsTable.writeCell("description");
			odsTable.writeCell(prot.get_description());
			// odsTable.writeCell("Sequence");
			odsTable.writeCell(peptideThisProt.getSequence());
			// odsTable.writeCell("Modifs");
			odsTable.writeCell(peptideThisProt
					.get_Modifs_formater_without_isotope());
			// odsTable.writeCell("Start");
			odsTable.writeCell(peptideThisProt.get_start());
			// odsTable.writeCell("Stop");
			odsTable.writeCell(peptideThisProt.get_stop());
			// odsTable.writeCell("MH+ theo");
			odsTable.writeCell(peptideThisProt.get_mhplus_theo());

			// S.append("\t" + pep.get_mhplus_theo());
			// odsTable.writeCell(peptide.get_deltamass());

			// nbSpectra = pepMass.getNumberOfPsm();
			// S.append("\t" + pep.get_deltamass());
			// odsTable.writeCell(((peptide.get_deltamass() / peptide
			// .get_mhplus_theo()) * 1000000));

			logger.debug("writeOnePeptideMassPerProt end");
		} catch (XMLStreamException e) {
			String message = "XMLStreamException in writeOnePeptideMassPerProt "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception in writeOnePeptideMassPerProt "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}
}
