/*******************************************************************************
 * Copyright (c) 2013 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 *
 * This file is part of XTandemPipeline.
 *
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contributors:
 *     Benoit Valot <Benoit.Valot@moulon.inra.fr>
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr>
 ******************************************************************************/

package fr.inra.pappso.xtandempipeline.output.SpreadSheets;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.stream.XMLStreamException;

import org.apache.log4j.Logger;

import fr.inra.pappso.libodsstream.OdsDocWriter;
import fr.inra.pappso.libodsstream.OdsDocWriterInterface;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.Match;
import fr.inra.pappso.xtandempipeline.class_msms.Modifs;
import fr.inra.pappso.xtandempipeline.class_msms.MsRun;
import fr.inra.pappso.xtandempipeline.class_msms.GroupingProtein.PeptideMass;
import fr.inra.pappso.xtandempipeline.class_msms.GroupingProtein.PeptideMassSet;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;
import fr.inra.pappso.xtandempipeline.grouping.Group;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScan;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSetPhospho;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSetProt;
import fr.inra.pappso.xtandempipeline.grouping.SubGroup;
import fr.inra.pappso.xtandempipeline.grouping.interfaces.GrpPhosphoModif;

public class OdsPhosphoStatsOutput {
	private static final Logger logger = Logger
			.getLogger(OdsPhosphoStatsOutput.class);

	protected OdsDocWriterInterface odsTable;

	private Hashtable<String, Integer> aa2count = new Hashtable<String, Integer>();

	private int countHashSampleScan = 0;

	protected OdsPhosphoStatsOutput() {

	}

	public OdsPhosphoStatsOutput(OutputStream outputStream)
			throws MSMSException {
		try {
			odsTable = new OdsDocWriter(outputStream);
			odsTable.writeSheet("phosho stats");
		} catch (IOException e) {
			String message = "IOException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (XMLStreamException e) {
			String message = "XMLStreamException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (DatatypeConfigurationException e) {
			String message = "DatatypeConfigurationException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	public OdsPhosphoStatsOutput(OdsDocWriterInterface odsTable)
			throws MSMSException {
		try {
			this.odsTable = odsTable;
			odsTable.writeSheet("phosho stats");
		} catch (XMLStreamException e) {
			String message = "XMLStreamException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	public void close() throws MSMSException {
		try {
			odsTable.close();
		} catch (Exception e) {
			String message = "Exception closing OdsPhosphoStatsOutput "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	private void writeHeaders(Identification ident) throws MSMSException {
		try {
			// Group Peptide ID Sample Scan Rt Sequence (top) Modifs (Top) Best
			// Position in peptide Phosphopeptide All Position in sequence
			// Sub-groups Ids Best E-value Charge MH+ Obs MH+ theo DeltaMH+
			// Delta-ppm
			odsTable.writeLine();
		} catch (XMLStreamException e) {
			String message = "XMLStreamException in writeHeaders() "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception in writeHeaders() " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}

	}

	private void writeCount() throws MSMSException {
		try {
			odsTable.writeLine();
			odsTable.writeCell("total number of sample/scan");
			odsTable.writeCell(this.countHashSampleScan);
			odsTable.writeLine();
			odsTable.writeCell("total number of phosphorylations per residue");

			for (String aa : this.aa2count.keySet()) {
				odsTable.writeLine();
				odsTable.writeCell(aa);
				odsTable.writeCell(this.aa2count.get(aa));
			}

		} catch (XMLStreamException e) {
			String message = "XMLStreamException in writeHeaders() "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception in writeHeaders() " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	public void write(Identification ident) throws MSMSException {
		try {
			countHashSampleScan = 0;
			aa2count = new Hashtable<String, Integer>();
			this.writeHeaders(ident);
			for (Group group : ident.getGrouping().getGroupList()) {
				PeptideMassSet pepMassSet = new PeptideMassSet();

				// logger.debug("pepMassSet size : " + pepMassSet.size());

				// logger.debug("pepMassSet numbering");
				HashMap<PeptideMass, HashSet<Peptide>> hashPeptideMass2Peptides = pepMassSet
						.numberingPhospho(group);

				HashSet<HashSampleScan> uniqueHashSampleScan = new HashSet<HashSampleScan>();
				for (SubGroup sg : group.getSubGroupSet().getSubGroupList()) {
					for (HashSampleScanSetProt hashPprot : sg
							.getHashSampleScanSetProtSet()) {
						HashSampleScanSetPhospho hashPhospho = (HashSampleScanSetPhospho) hashPprot;

						HashSet<PeptideMass> uniquePepMass = new HashSet<PeptideMass>();

						for (Peptide peptide : ((Match) hashPhospho
								.getGrpProtMatch())
								.getValidPeptideInPhosphoIsland((HashSampleScanSetPhospho) hashPhospho)) {

							PeptideMass pepMass = pepMassSet
									.getInstance(peptide);
							// pepMass.addSubGroup(sg);
							// HashPeptide p = peptide.getHashPeptide();
							if (uniquePepMass.add(pepMass)) {

								for (HashSampleScan scan : pepMass
										.getHashSampleScanSet()) {

									if (uniqueHashSampleScan.add(scan)) {

										this.writeOneHashSampleScan(ident,
												group, sg, hashPhospho,
												pepMass, scan,
												hashPeptideMass2Peptides
														.get(pepMass));
									}
								}
							}

						}
					}
				}

			}
			this.writeCount();

		} catch (Exception e) {
			String message = "Exception in write(Identification ident) "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	private void writeOneHashSampleScan(Identification ident, Group group,
			SubGroup sg, HashSampleScanSetPhospho hashPhospho,
			PeptideMass pepMass, HashSampleScan scan,
			HashSet<Peptide> peptideSet) throws MSMSException {
		try {
			// logger.debug("writeOneHashSampleScan begin " + group + pepMass);
			this.countHashSampleScan++;
			HashSet<Peptide> peptideSetScan = new HashSet<Peptide>();
			MsRun msRun = (MsRun) scan.getGrpMsSample();
			Peptide bestPeptide = null;
			for (Peptide peptide : peptideSet) {
				if (peptide.getMsRun().equals(msRun)) {
					peptideSetScan.add(peptide);
					if (peptide.getHashSampleScan() == scan) {
						if (bestPeptide == null) {
							bestPeptide = peptide;
						} else {
							if (bestPeptide.get_evalue() > peptide.get_evalue()) {
								bestPeptide = peptide;
							}
						}
					}
				}
			}
			if (bestPeptide == null) {
				// throw new MSMSException("peptideScan == null");
				return;
			}

			Collection<GrpPhosphoModif> phosphoModifs = bestPeptide
					.getGrpPhosphoModifs();

			for (GrpPhosphoModif phosphoModif : phosphoModifs) {
				Modifs modif = (Modifs) phosphoModif;
				// obsPhosphoPos.add(modif.get_posi());

				String currentAA = modif.get_AA();
				/*
				 * if ((!currentAA.equals("S") && (!currentAA.equals("T")) &&
				 * (!currentAA .equals("Y")))) { logger.debug(currentAA + " " +
				 * bestPeptide.get_sequence_formater() + " " +
				 * modif.get_formater_value()); }
				 */
				Integer currentCount = this.aa2count.get(currentAA);
				if (currentCount == null) {
					this.aa2count.put(currentAA, new Integer(1));
					// logger.debug("new count " + currentAA);
				} else {
					currentCount++;
					this.aa2count.put(currentAA, currentCount);
					// logger.debug("count ++ " + currentCount + " " +
					// currentAA);
				}
			}
			// odsTable.writeCell("Best Position in sequence");
			// odsTable.writeCell(Utils.collectionToString(obsPhosphoPos));

			// odsTable.writeCell(pepMass.getSequenceLi());

			// logger.debug("writeOneHashSampleScan end");
		} catch (Exception e) {
			String message = "Exception in writeOneHashSampleScan "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	public void writeStats() {
		// TODO Auto-generated method stub

	}
}
