package fr.inra.pappso.xtandempipeline.thread;

import java.io.File;
import java.io.FileOutputStream;
import java.util.ArrayList;

import org.apache.log4j.Logger;

import fr.inra.pappso.csv2ods.TsvWriterMultiSheet;
import fr.inra.pappso.csv2ods.TsvWriterOneSheet;
import fr.inra.pappso.libodsstream.OdsDocWriter;
import fr.inra.pappso.libodsstream.OdsDocWriterInterface;
import fr.inra.pappso.xtandempipeline.XtandemPipelineSession;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.MsException.StopException;
import fr.inra.pappso.xtandempipeline.class_msms.config;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.ModifsList;
import fr.inra.pappso.xtandempipeline.filter_print.calculation_peptide_fdr_distribution;
import fr.inra.pappso.xtandempipeline.filter_print.calculation_protein_fdr_distribution;
import fr.inra.pappso.xtandempipeline.filter_print.filter_isotopic_modifs;
import fr.inra.pappso.xtandempipeline.filter_print.print_fasta;
import fr.inra.pappso.xtandempipeline.output.FastaOutputStream;
import fr.inra.pappso.xtandempipeline.output.MassChroQmlOutput;
import fr.inra.pappso.xtandempipeline.output.MassChroqPrmOutput;
import fr.inra.pappso.xtandempipeline.output.PepnovoMlOutput;
import fr.inra.pappso.xtandempipeline.output.ProticDbMlOutput;
import fr.inra.pappso.xtandempipeline.output.SpreadSheets.OdsGeneralInformationOutput;
import fr.inra.pappso.xtandempipeline.output.SpreadSheets.OdsMsSampleInformationOutput;
import fr.inra.pappso.xtandempipeline.output.SpreadSheets.OdsPeptideOutput;
import fr.inra.pappso.xtandempipeline.output.SpreadSheets.OdsPeptidePosOutput;
import fr.inra.pappso.xtandempipeline.output.SpreadSheets.OdsProteinOutput;
import fr.inra.pappso.xtandempipeline.output.SpreadSheets.OdsSpectraOutput;

public class printing_base extends XtpProcess {
	protected static final Logger logger = Logger
			.getLogger(printing_base.class);

	private File out;

	protected ArrayList<Identification> identifications = XtandemPipelineSession
			.getInstance().getIdentificationList();

	private print_fasta fasta_type;

	private ModifsList isotopic_mods;

	protected boolean isCsv;

	protected boolean isExpertMode = false;

	public printing_base(config conf) {
		super(conf);
		isotopic_mods = new ModifsList();
		this.fasta_type = null;
		this.set_max_progress(8);
	}

	protected void processing() throws Exception {

		// Ecrire le process et penser à relever des StopException
		logger.info("Printing to other format");

		// msms_output output = new msms_output();
		isExpertMode = conf.get_export_expert_spreadsheet();
		if (conf.get_export_txt()) {
			this.set_view("Printing text tabulated file results");
			this.isCsv = true;
			this.spreadSheetReport();
		}

		if (conf.get_export_ods()) {
			this.set_view("Printing Open Document Spreadsheet file results");
			this.isCsv = false;
			this.spreadSheetReport();
		}

		if (conf.get_export_proticdbml()) {
			logger.debug("conf.get_export_proticdbml()");
			this.set_view("Printing PROTICdbML file results");

			FileOutputStream outputStream = new FileOutputStream(new File(this
					.getfileout().getAbsolutePath() + "_protic.xml"));

			ProticDbMlOutput protidDbMlOutput = new ProticDbMlOutput();
			protidDbMlOutput.open(outputStream);
			// massChroQmlOutput.write(outputStream);
			for (Identification ident : this.identifications) {
				protidDbMlOutput.write(ident);
			}
			protidDbMlOutput.close();
			logger.debug("protidDbMlOutput.close()s");

			// proticdbml protic = new proticdbml(new File(this.getfileout()
			// .getAbsolutePath() + "_protic.xml"));
			// for (Identification ident : this.identifications) {
			// protic.add_identification(ident);
			// }
		}

		// Verification pas arrete
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		if (conf.get_export_fasta()) {
			logger.debug("conf.get_export_fasta()");
			this.set_view("Printing fasta file protein result");
			// HashSet<Protein> prot_deja_ecris = new HashSet<Protein>();
			FileOutputStream outputStream = new FileOutputStream(new File(this
					.getfileout().getAbsolutePath() + ".fasta"));

			FastaOutputStream ofasta = new FastaOutputStream(outputStream);
			// output.open(this.getfileout().getAbsolutePath() + ".fasta");
			this.fasta_type = this.fasta_export_choice(ofasta);
			for (Identification ident : this.identifications) {
				fasta_type.print(ident);
			}
			ofasta.close();
			outputStream.close();
		}
		// Verification pas arrete
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		if (conf.get_export_pepnovo()) {
			logger.debug("conf.get_export_pepnovo()");
			this.set_view("Printing PepNovo xml file results");

			FileOutputStream outputStream = new FileOutputStream(new File(this
					.getfileout().getAbsolutePath() + "_pepnovo.xml"));

			PepnovoMlOutput pepnovoOutput = new PepnovoMlOutput();
			pepnovoOutput.open(outputStream);
			// massChroQmlOutput.write(outputStream);
			for (Identification ident : this.identifications) {
				pepnovoOutput.write(ident);
			}
			pepnovoOutput.close();
		}
		// Verification pas arrete
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		// Print de fdr...
		if ((conf.get_Export_fdr()) & (this.identifications.size() == 1)) {
			this.set_view("Printing FDR");
			OdsDocWriterInterface odsDoc;
			if (conf.get_export_ods()) {
				odsDoc = new OdsDocWriter(this.getfileout().getAbsolutePath()
						+ "_fdr.ods");
			} else {
				odsDoc = new TsvWriterMultiSheet(new File(this.getfileout()
						.getAbsolutePath() + "_fdr"));

			}

			calculation_peptide_fdr_distribution calc = new calculation_peptide_fdr_distribution();
			calc.execute(this.identifications.get(0).get_protein_liste());
			calc.print_peptide_fdr_distribution(odsDoc);
			// output.close();

			// output.open(this.getfileout().getAbsolutePath()
			// + "_fdr_protein.txt");
			calculation_protein_fdr_distribution calc2 = new calculation_protein_fdr_distribution();
			calc2.execute(this.identifications.get(0).get_protein_liste());
			calc2.print_protein_fdr_distribution(odsDoc);
			// output.close();
			odsDoc.close();
		}
		// Verification pas arrete
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		if ((conf.get_export_masschroq()) & (this.identifications.size() == 1)) {
			this.set_view("Printing MassChroQ xml file results");
			Identification ident = this.identifications.get(0);
			// modification isotopic, filter isotope labels
			ident.filtered_match(new filter_isotopic_modifs(this.isotopic_mods));

			MassChroQmlOutput massChroQmlOutput = new MassChroQmlOutput(
					this.isotopic_mods, ident);
			FileOutputStream outputStream = new FileOutputStream(new File(this
					.getfileout().getAbsolutePath() + ".masschroqML"));

			massChroQmlOutput.write(outputStream);
		}

		if ((conf.get_export_masschroqprm())
				& (this.identifications.size() == 1)) {
			this.set_view("Printing MassChroqPrm xml peptide list file");
			Identification ident = this.identifications.get(0);
			// modification isotopic
			ident.filtered_match(new filter_isotopic_modifs(this.isotopic_mods));

			MassChroqPrmOutput massChroQmlOutput = new MassChroqPrmOutput(
					this.isotopic_mods, ident);
			FileOutputStream outputStream = new FileOutputStream(new File(this
					.getfileout().getAbsolutePath() + ".masschroqPRM"));

			massChroQmlOutput.write(outputStream);
		}

		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

	}

	protected void spreadSheetReport() throws Exception {
		logger.info("spreadSheetReport begin");

		this.set_view("Printing protein file results");

		OdsDocWriter odsDoc = null;
		try {
			if (this.isCsv) {

			} else {
				odsDoc = new OdsDocWriter(this.getfileout().getAbsolutePath()
						+ ".ods");
			}
			logger.debug("Print XTP general informations");
			this.printGeneralInformations(odsDoc);

			logger.debug("Print new proteins");
			this.printProteinList(odsDoc);
			logger.debug("Print new peptide");
			this.printPeptideList(odsDoc);
			logger.debug("Print new spectra compar");
			this.printComparSpectra(odsDoc);
			logger.debug("Print new spectra specific");
			this.printComparSpecificSpectra(odsDoc);
			logger.debug("Print new spectra unique seq compar");
			this.printComparUniqueSpectra(odsDoc);
			logger.debug("Print new spectra specific unique seq compar");
			this.printComparSpecificUniqueSpectra(odsDoc);
			logger.debug("Print new compar pai");
			this.printComparPai(odsDoc);

			if (this.isExpertMode) {
				logger.debug("Print new spectra (expert)");
				this.printSpectraList(odsDoc);
				logger.debug("Print new peptide/prot (expert)");
				this.printPeptideProtList(odsDoc);
			}

			logger.debug("Print XTP MS sample informations");
			this.printMsSamplesDetails(odsDoc);
			// odsDoc.close();
		} catch (Exception e) {
			String message = "Error in spreadSheetReport :\n" + e.getMessage();
			logger.error(message);
			throw new MSMSException(message, e);
		} finally {
			if (odsDoc != null) {
				logger.debug("Closing odsDoc");
				odsDoc.close();
			}
		}
		this.add_current_progress();
	}

	protected void printComparSpecificUniqueSpectra(OdsDocWriterInterface odsDoc)
			throws Exception {

	}

	protected void printComparUniqueSpectra(OdsDocWriterInterface odsDoc)
			throws Exception {

	}

	protected void printGeneralInformations(OdsDocWriterInterface odsDoc)
			throws Exception {
		logger.debug("printGeneralInformations begin");

		// Verification pas arrete
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		this.set_view("Printing general informations about X!TandemPipeline process");
		if (this.isCsv) {
			odsDoc = new TsvWriterOneSheet(new File(this.getfileout()
					.getAbsolutePath() + "_infos.txt"));
		} else {
		}

		OdsGeneralInformationOutput writer = new OdsGeneralInformationOutput(
				odsDoc);

		writer.writeXtpSessionInformations(XtandemPipelineSession.getInstance()
				.getDataTypeBase(), XtandemPipelineSession.getInstance()
				.getIdentificationList());
		logger.debug("printGeneralInformations end");

		if (this.isCsv) {
			odsDoc.close();
			logger.debug("csv file closed "
					+ this.getfileout().getAbsolutePath()
					+ "_general_infos.txt");
		}

	}

	protected void printMsSamplesDetails(OdsDocWriterInterface odsDoc)
			throws Exception {
		logger.debug("printMsSamplesDetails begin");

		// Verification pas arrete
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		this.set_view("Printing informations about MS samples and identification process");
		if (this.isCsv) {
			odsDoc = new TsvWriterOneSheet(new File(this.getfileout()
					.getAbsolutePath() + "_infos.txt"));
		} else {
		}

		OdsMsSampleInformationOutput writer = new OdsMsSampleInformationOutput(
				odsDoc);

		writer.writeXtpSessionInformations(XtandemPipelineSession.getInstance()
				.getDataTypeBase(), XtandemPipelineSession.getInstance()
				.getIdentificationList());
		logger.debug("printXtpRunningDetails end");

		if (this.isCsv) {
			odsDoc.close();
			logger.debug("csv file closed "
					+ this.getfileout().getAbsolutePath()
					+ "_ms_samples_infos.txt");
		}

	}

	protected void printComparPai(OdsDocWriterInterface odsDoc)
			throws Exception {

	}

	protected void printComparSpecificSpectra(OdsDocWriterInterface odsDoc)
			throws Exception {
	}

	protected void printComparSpectra(OdsDocWriterInterface odsDoc)
			throws Exception {
	}

	private print_fasta fasta_export_choice(FastaOutputStream ofasta) {
		if (this.fasta_type == null) {
			this.fasta_type = new print_fasta();
			logger.debug("fasta export reinitialise");
		}
		this.fasta_type.setOut(ofasta);
		return (this.fasta_type);
	}

	synchronized public void setfileout(File out) {
		this.out = out;
	}

	synchronized public File getfileout() {
		return this.out;
	}

	synchronized public void setIsotopic_mods(ModifsList isotopic_mods) {
		this.isotopic_mods = isotopic_mods;
	}

	synchronized public ModifsList getIsotopic_mods() {
		return isotopic_mods;
	}

	synchronized public void setFasta_type(print_fasta fasta_type)
			throws MSMSException {
		logger.debug("setFasta_type");
		if (fasta_type == null) {
			String message = "fasta_type == null";
			logger.error(message);
			throw new MSMSException(message);

		}
		this.fasta_type = fasta_type;
		logger.debug(this.fasta_type.toString());
	}

	public print_fasta getFasta_type() {
		return fasta_type;
	}

	protected void printProteinList(OdsDocWriterInterface odsDoc)
			throws Exception {
		logger.debug("printProteinList begin");

		// Verification pas arrete
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		this.set_view("Printing protein list results");
		if (this.isCsv) {
			odsDoc = new TsvWriterOneSheet(new File(this.getfileout()
					.getAbsolutePath() + "_proteins.txt"));
		} else {
		}

		OdsProteinOutput writer = new OdsProteinOutput(odsDoc);
		// output.open(this.getfileout().getAbsolutePath() + "_compar.txt");
		for (Identification ident : this.identifications) {
			this.writeMsSampleName(ident, odsDoc);
			writer.write(ident);
		}

		if (this.isCsv) {
			odsDoc.close();
			logger.debug("csv file closed "
					+ this.getfileout().getAbsolutePath() + "_proteins.txt");
			odsDoc = null;
		}
		logger.debug("printProteinList end");
	}

	protected void printSpectraList(OdsDocWriterInterface odsDoc)
			throws Exception {
		logger.debug("printSpectraList begin");

		// Verification pas arrete
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		this.set_view("Printing spectra list results");
		if (this.isCsv) {
			odsDoc = new TsvWriterOneSheet(new File(this.getfileout()
					.getAbsolutePath() + "_spectra.txt"));
		} else {
		}

		OdsSpectraOutput writer = new OdsSpectraOutput(odsDoc);
		// output.open(this.getfileout().getAbsolutePath() + "_compar.txt");
		for (Identification ident : this.identifications) {
			this.writeMsSampleName(ident, odsDoc);
			writer.write(ident);
		}
		logger.debug("printSpectraList end");

		if (this.isCsv) {
			odsDoc.close();
			logger.debug("csv file closed "
					+ this.getfileout().getAbsolutePath() + "_spectra.txt");
		}

	}

	protected void printPeptideProtList(OdsDocWriterInterface odsDoc)
			throws Exception {
		logger.debug("printPeptideProtList begin");

		// Verification pas arrete
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		this.set_view("Printing peptide/prot list results");
		if (this.isCsv) {
			odsDoc = new TsvWriterOneSheet(new File(this.getfileout()
					.getAbsolutePath() + "_peptides_pos.txt"));
		} else {
		}

		OdsPeptidePosOutput writer = new OdsPeptidePosOutput(odsDoc);
		// output.open(this.getfileout().getAbsolutePath() + "_compar.txt");
		for (Identification ident : this.identifications) {
			this.writeMsSampleName(ident, odsDoc);
			writer.write(ident);
		}
		logger.debug("printPeptideProtList end");

		if (this.isCsv) {
			odsDoc.close();
			logger.debug("csv file closed "
					+ this.getfileout().getAbsolutePath() + "_peptides_pos.txt");
		}

	}

	protected void writeMsSampleName(Identification ident,
			OdsDocWriterInterface odsDoc) throws Exception {

	}

	protected void printPeptideList(OdsDocWriterInterface odsDoc)
			throws Exception {
		logger.debug("printPeptideList begin");

		// Verification pas arrete
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		this.set_view("Printing peptide list results");
		if (this.isCsv) {
			odsDoc = new TsvWriterOneSheet(new File(this.getfileout()
					.getAbsolutePath() + "_peptide_list.txt"));
		} else {
		}

		OdsPeptideOutput writer = new OdsPeptideOutput(odsDoc);
		// output.open(this.getfileout().getAbsolutePath() + "_compar.txt");
		for (Identification ident : this.identifications) {
			this.writeMsSampleName(ident, odsDoc);
			writer.write(ident);
		}
		logger.debug("printPeptideList end");

		if (this.isCsv) {
			odsDoc.close();
			logger.debug("csv file closed "
					+ this.getfileout().getAbsolutePath() + "_peptide_list.txt");
		}
	}
}
