package fr.inra.pappso.xtandempipeline.ui.swt;

import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.ProgressBar;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import fr.inra.pappso.xtandempipeline.thread.XtpProcess;

public class Running {
	private static final Logger logger = Logger.getLogger(Running.class);
	protected XtpProcess encapsulatedProcess;

	private Shell shell;

	protected Main_Windows controlleur;

	private Text text;

	private Label memory;

	private ProgressBar progress;

	private Image[] animatedImage = new Image[8];

	private Label anim;

	private int currentAnim = 0;

	public Running(Main_Windows controlleur, XtpProcess pro, boolean bigdim) {
		this.controlleur = controlleur;
		encapsulatedProcess = pro;
		shell = new Shell(controlleur.getDisplay(), SWT.APPLICATION_MODAL
				| SWT.SHELL_TRIM);
		init();
		if (bigdim)
			shell.setSize(450, 500);
		else
			shell.setSize(350, 200);
		shell.setImages(controlleur.getImages());
		shell.open();
		logger.debug("OPen");
		while (!shell.isDisposed()) {
			if (!controlleur.getDisplay().readAndDispatch()) {
				controlleur.getDisplay().sleep();
			}
		}
	}

	private void init() {
		GridLayout layout = new GridLayout(2, false);
		layout.marginLeft = layout.marginTop = layout.marginRight = layout.marginBottom = 10;
		layout.verticalSpacing = 10;
		shell.setLayout(layout);

		this.progress = new ProgressBar(shell, SWT.HORIZONTAL);
		progress.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false,
				1, 1));

		// on charge les images
		for (int i = 1; i < 9; i++)
			animatedImage[i - 1] = new Image(shell.getDisplay(),
					Running.class.getResourceAsStream("/resources/images/anim_"
							+ i + ".png"));
		this.anim = new Label(shell, SWT.NONE);
		anim.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, false, false));
		anim.setImage(animatedImage[0]);

		text = new Text(shell, SWT.MULTI | SWT.WRAP);
		Label separator = new Label(shell, SWT.SEPARATOR | SWT.HORIZONTAL);
		memory = new Label(shell, SWT.NONE);

		Button cancelButton = new Button(shell, SWT.PUSH);
		cancelButton.setText("Cancel");
		cancelButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				logger.debug("Stop");
				encapsulatedProcess.setStop(true);
			}
		});
		text.setText("No identification result");
		GridData gridData = new GridData(GridData.FILL_BOTH);
		gridData.horizontalSpan = 2;
		text.setLayoutData(gridData);
		gridData = new GridData(GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 2;
		separator.setLayoutData(gridData);
		memory.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		// on met à jour
		this.progress.setMinimum(0);
		this.refreshDisplay();

		// on demarre le process
		final int time = 150;
		encapsulatedProcess.setDaemon(true);
		encapsulatedProcess.start();
		final Runnable timer = new Runnable() {
			public void run() {
				if (Running.this.shell.isDisposed()) {
					encapsulatedProcess.setStop(true);
				} else if (encapsulatedProcess.isAlive()) {
					controlleur.getDisplay().timerExec(time, this);
					Running.this.refreshDisplay();
				} else {
					Running.this.stopping();
				}
			}
		};
		controlleur.getDisplay().timerExec(time, timer);
	}

	synchronized protected void stopping() {
		if (!shell.isDisposed())
			shell.dispose();
	}

	synchronized protected void refreshDisplay() {
		// animation de l'image
		currentAnim++;
		if (currentAnim == 8) {
			currentAnim = 0;
		}
		anim.setImage(animatedImage[currentAnim]);

		// on ne met à jour l'interface complete que toute les 150*4=550ms
		if (currentAnim == 0 || currentAnim == 4) {
			Runtime run = Runtime.getRuntime();
			long used = (run.totalMemory() - run.freeMemory()) / 1000000;
			long total = run.totalMemory() / 1000000;
			String S = used + "/" + total + "Mo";
			memory.setText(S);
			text.setText(encapsulatedProcess.getView());
			this.progress.setMaximum(encapsulatedProcess.get_max_progress());
			this.progress.setSelection(encapsulatedProcess
					.get_current_progress());
		}
	}
}
