package fr.inra.pappso.xtandempipeline.class_msms;

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.regex.*;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.XtandemPipelineSession;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;
import fr.inra.pappso.xtandempipeline.grouping.HashProtein;
import fr.inra.pappso.xtandempipeline.grouping.interfaces.GrpProtein;

public class Protein implements GrpProtein {
	private static final Logger logger = Logger.getLogger(Protein.class);

	private float mw;

	private Integer phosphoId;

	private int theoreticalTrypticPeptideNumber;

	private String description;

	private String sequence;

	private Hashtable<MsRun, Float> sample_evalue = new Hashtable<MsRun, Float>();

	private float evalue;

	private Database URL;

	private String dbxref_type;

	private String dbxref_accession;

	private boolean isReversed;

	// constructeur
	public Protein() {
		this.sequence = "";
		this.dbxref_type = "no";
		this.isReversed = false;
	}

	public Hashtable<MsRun, Float> get_hash_sample_evalue() {
		return (this.sample_evalue);
	}

	public void set_sequence(String f) {
		this.sequence = wash_sequence(f);
		this.mw = this.determined_mw();
		this.theoreticalTrypticPeptideNumber = this
				.getTheoreticalTrypticPeptideNumber();
	}

	public void set_description(String S) {

		// If contained reversed option
		S = S.trim();
		S = S.replaceAll("\n", "");
		S = S.replaceAll("\r", "");
		if (S.matches(".*\\:reversed$")) {
			logger.debug("Corrected reverse protein description");
			this.description = S.replaceFirst("\\:reversed$", "");
			this.setReversed(true);
		} else if (S.split(" ")[0].matches(".*\\|reversed$")) {
			this.isReversed = true;
			this.description = S;
		} else
			this.description = S;
		this.search_dbxref();
		// logger.debug("set_description end");
		//[J3KPE3, P63244]
		//[A0A087X0X3, P52272, P52272-2]
	}

	public void set_evalue(MsRun Sample, Float Evalue) {
		if (sample_evalue.containsKey(Sample) == false)
			sample_evalue.put(Sample, Evalue);
	}

	public void set_evalue(Float Evalue) {
		this.evalue = Evalue;
	}

	public void corrected_evalue(MsRun Sample, Float Evalue) {
		sample_evalue.put(Sample, Evalue);
	}

	public void setDatabase(Database S) {
		this.URL = S;
	}

	public boolean hasSequence() {
		if (this.sequence.equals(""))
			return (false);
		else
			return (true);
	}

	public String get_sequence() {
		return (this.sequence);
	}

	public Database getDatabase() {
		return (this.URL);
	}

	public float get_mw() {
		return (mw);
	}

	public String get_description() {
		return (description);
	}

	@Override
	public String getAccession() {
		if (description.contains(" ")) {
			String[] S = description.split(" ");
			return (S[0]);
		}
		else {
			return description;
		}
	}

	public String get_dbxref_accession() {
		return (this.dbxref_accession);
	}

	public String get_dbxref_type() {
		return (this.dbxref_type);
	}

	public float get_evalue(MsRun sample) {
		if (sample_evalue.containsKey(sample)) {
			return (sample_evalue.get(sample));
		} else {
			return (0);
		}
	}

	public float get_evalue() {
		return (this.evalue);
	}

	public boolean protein_evalue_more_than(float prot_evalue) {
		boolean conserve = false;
		Enumeration<Float> E = sample_evalue.elements();
		while (E.hasMoreElements()) {
			float m = E.nextElement();
			if (m < prot_evalue)
				conserve = true;
		}
		return (conserve);
	}

	private String wash_sequence(String seq) {
		seq = seq.toUpperCase();
		seq = seq.replaceAll("[^GASPVTLINDKQEMHFRCYWU-]", "");
		return (seq);
	}

	private void search_dbxref() {
		String accession = this.getAccession();
		String[] access = accession.split("\\|");
		// if (access.length == 1)
		String atg = "^[Aa][Tt][MmCc1-5][Gg]\\d{5}\\.?\\d?$";
		String ncbi_gi = "^[0-9]{5,8}$";
		String swiss_prot = "^P[A-Z0-9]{5}$";
		String trembl = "^[QOA][A-Z0-9]{5}$";
		String ref = "^[NZ]P\\_[0-9]{5,8}$";

		for (int i = (access.length - 1); i >= 0; i--) {
			if (Pattern.matches(atg, access[i])) {
				String temp[] = access[i].split("\\.");
				this.dbxref_accession = temp[0];
				this.dbxref_type = "AGI_LocusCode";
			}
			if (Pattern.matches(ncbi_gi, access[i])) {
				this.dbxref_accession = access[i];
				this.dbxref_type = "NCBI_gi";
			}
			if (Pattern.matches(swiss_prot, access[i])) {
				this.dbxref_accession = access[i];
				this.dbxref_type = "Swiss-Prot";
			}
			if (Pattern.matches(trembl, access[i])) {
				this.dbxref_accession = access[i];
				this.dbxref_type = "TrEMBL";
			}
			if (Pattern.matches(ref, access[i])) {
				this.dbxref_accession = access[i];
				this.dbxref_type = "ref";
			}
		}

	}

	/**
	 * get the theoretical number of tryptic peptides between 800 and 2500
	 * Daltons for one protein
	 *
	 * @return integer number of possible tryptic peptides
	 */
	private int getTheoreticalTrypticPeptideNumber() {
		int count = 0;
		Hashtable<String, String> aa = new Hashtable<String, String>();
		aa.put("G", "57");
		aa.put("A", "71");
		aa.put("S", "87");
		aa.put("P", "97");
		aa.put("V", "99");
		aa.put("T", "101");
		aa.put("L", "113");
		aa.put("I", "113");
		aa.put("N", "114");
		aa.put("D", "115");
		aa.put("K", "128");
		aa.put("Q", "128");
		aa.put("E", "129");
		aa.put("M", "131");
		aa.put("H", "137");
		aa.put("F", "147");
		aa.put("R", "156");
		aa.put("C", "103");
		aa.put("Y", "163");
		aa.put("W", "186");
		aa.put("-", "0");

		String seq = this.get_sequence();
		seq = seq.replaceAll("K", "K_");
		seq = seq.replaceAll("R", "R_");
		String[] S_split = seq.split("[_]");
		for (int o = 0; o < S_split.length; o++) {
			int mw_calc = 0;
			for (int i = 0; i < S_split[o].length(); i++) {
				String S = "" + S_split[o].charAt(i);
				if (aa.containsKey(S)) {
					mw_calc += Integer.parseInt((String) aa.get(S));
				}
			}
			if (mw_calc > 800)
				if (mw_calc < 2500)
					count++;
		}

		return (count);
	}

	private float determined_mw() {
		Hashtable<String, String> aa = new Hashtable<String, String>();
		aa.put("G", "57");
		aa.put("A", "71");
		aa.put("S", "87");
		aa.put("P", "97");
		aa.put("V", "99");
		aa.put("T", "101");
		aa.put("L", "113");
		aa.put("I", "113");
		aa.put("N", "114");
		aa.put("D", "115");
		aa.put("K", "128");
		aa.put("Q", "128");
		aa.put("E", "129");
		aa.put("M", "131");
		aa.put("H", "137");
		aa.put("F", "147");
		aa.put("R", "156");
		aa.put("C", "103");
		aa.put("Y", "163");
		aa.put("W", "186");
		aa.put("-", "0");

		int mw_calc = 0;
		for (int i = 0; i < this.sequence.length(); i++) {
			String S = "" + this.sequence.charAt(i);
			if (aa.containsKey(S)) {
				mw_calc += Integer.parseInt((String) aa.get(S));
			}
		}

		mw_calc = mw_calc / 100;
		float mw_float = (float) mw_calc / 10;

		return (mw_float);
	}

	@Override
	public int compareTo(GrpProtein prot) {
		return (this.getAccession().compareTo(prot.getAccession()));
	}

	public HashProtein getHashProtein() throws MSMSException {
		String accession = this.getAccession();
		if (accession == null) {
			String error = "accession is null";
			logger.debug(error);
			throw new MSMSException(error);
		}
		if (accession.isEmpty()) {
			String error = "accession is empty";
			logger.debug(error);
			throw new MSMSException(error);
		}
		return XtandemPipelineSession.getInstance().getHashProteinInstance(
				accession);
	}

	public boolean isReversed() {
		return isReversed;
	}

	public void setReversed(boolean rev) {
		this.isReversed = rev;
		// forced reverse annotation on protein accession
		if (isReversed) {
			if (description.contains(" ")) {
				this.description = description.replaceFirst(" ", "|reversed ");
			} else {
				this.description = this.description.concat("|reversed");
			}
		}
	}

	@Override
	public int getNumberOfTrypticPeptidesForPAI() {
		return this.theoreticalTrypticPeptideNumber;
	}

	/**
	 * get the protein identifier of this protein usefull in the PhosphoIsland
	 * context to get a unique PhosphoIsland ID
	 *
	 * @return int
	 * @throws MSMSException
	 */
	public int getPhosphoId() throws MSMSException {
		if (this.phosphoId == null) {
			String error = "phosphoId is null";
			logger.debug(error);
			throw new MSMSException(error);
		}
		return this.phosphoId.intValue();
	}

	/**
	 * set the protein identifier of this protein usefull in the PhosphoIsland
	 * context to get a unique PhosphoIsland ID the developper MUST give an id
	 * number for each protein of PhosphoIslands
	 *
	 * @param i
	 */
	public void setPhosphoId(int i) {
		if (this.phosphoId == null) {
			this.phosphoId = i;
		}

	}

	public void tryToPlacePeptide(Peptide pep) throws MSMSException {
		// logger.debug("tryToPlacePeptide begin");
		String pepStr = pep.getSequence();
		// logger.debug("tryToPlacePeptide " + pepStr + " on "
		// + this.getAccession());
		int fromIndex = 0;
		int start = this.sequence.indexOf(pepStr, fromIndex);
		if (start > -1) {
			pep.set_start(start + 1);
			pep.set_stop(start + pepStr.length());

		} else {
			// peptide does not occurs in sequence
			// check with L/I substitution :
			String pepLi = pepStr.replaceAll("L", "I");
			Pattern pattern = Pattern.compile(pepLi.replaceAll("I", "[LI]"));
			Matcher matcher = pattern.matcher(this.sequence);
			// Check all occurrences
			int i = 0;
			while (matcher.find()) {
				i++;
				pep.set_start(matcher.start() + 1);
				pep.set_stop(matcher.start() + pepStr.length());
				logger.debug("Start index: " + matcher.start());
				logger.debug(" End index: " + matcher.end());
				logger.debug(" Found: " + matcher.group());
			}
			// throw new MSMSException("choses");

			if (i == 0) {
				String message = "peptideLI " + pepStr
						+ " does not occur in protein " + this.getAccession();
				// multiple occurences !!!!
				logger.error(message);
				logger.error(this.sequence);
				throw new MSMSException(message);
			} else if (i > 1) {
				String message = "multiple occurences of peptideLI " + pepStr
						+ " in protein " + this.getAccession();
				// multiple occurences !!!!
				logger.warn(message);
			}

		}
		fromIndex = pep.get_stop();
		start = this.sequence.indexOf(pepStr, fromIndex);
		if (start > 0) {
			String message = "multiple occurences of peptide " + pepStr
					+ " in protein " + this.getAccession();
			// multiple occurences !!!!
			logger.warn(message);
		}
	}
}
